/*
 * icmp.c - send ICMP Echo packets
 * Copyright (c) 2005-2006, D.C. van Moolenbroek, XISE <dc@xise.nl>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <ipexport.h>
#include <icmpapi.h>

#include "icmp.h"

BOOL IcmpPing(DWORD ip, DWORD timeout, DWORD ttl, IcmpResult *res) {
  BYTE buf[sizeof(ICMP_ECHO_REPLY) + 8];
  PICMP_ECHO_REPLY reply;
  IP_OPTION_INFORMATION *opt, optinfo;
  HANDLE handle;
  DWORD num;

  ZeroMemory(buf, sizeof(buf));

  handle = IcmpCreateFile();
  if (handle == INVALID_HANDLE_VALUE) return FALSE;

  if (ttl > 0) {
    ZeroMemory(&optinfo, sizeof(optinfo));

    optinfo.Ttl = (UCHAR)ttl;

    opt = &optinfo;
  }
  else opt = NULL;

  num = IcmpSendEcho(handle, ip, "", 0, opt, buf, sizeof(buf), timeout);

  if (num > 0) {
    reply = (PICMP_ECHO_REPLY)buf;

    res->status = reply->Status;
    if (res->status >= IP_STATUS_BASE) res->status -= IP_STATUS_BASE;

    res->ip = reply->Address;
    res->time = reply->RoundTripTime;
    res->ttl = reply->Options.Ttl;
  }

  IcmpCloseHandle(handle);

  return num > 0;
}
