/*
 * icmp.h - send ICMP Echo packets
 * Copyright (c) 2005-2006, D.C. van Moolenbroek, XISE <dc@xise.nl>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _ICMP_H
#define _ICMP_H

typedef struct {
  DWORD status; // the status code (one of the IP_* codes from ipexport.h)
  DWORD ip;     // the IP address of the replying node
  DWORD time;   // round-trip time in milliseconds
  DWORD ttl;    // time-to-live of the reply packet
} IcmpResult;

/*
 * function:    IcmpPing
 * purpose:     send a ICMP echo packet to an IP, and wait for a reply
 * parameters:  ip        the IP address of the host to send the packet to
 *              timeout   number of milliseconds to wait for a reply
 *              ttl       the time-to-live value for the packet, 0 for default
 *              res       pointer to a structure which will be filled with
 *                        information from the (first) reply packet, if any
 * returns:     TRUE if a reply has been received; in this case, the structure
 *              pointed to by 'res' will be filled; FALSE otherwise.
 */
extern BOOL IcmpPing(DWORD ip, DWORD timeout, DWORD ttl, IcmpResult *res);

#endif /*_ICMP_H*/
