/*
 * mirc.c - communication with mIRC process
 * Copyright (c) 2005-2006, D.C. van Moolenbroek, XISE <dc@xise.nl>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#define WM_MCOMMAND (WM_USER + 200)

/* send a command to a mIRC window */
void SendMirc(HWND hwnd, char *data) {
  HANDLE hmap, hmutex;
  char *ptr;

  // create the filemapping object
  if ((hmap = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE,
   0, 1024, "mIRC")) == NULL) return;

  // at this point we SHOULD check GetLastError() to see if it's set to
  // ERROR_ALREADY_EXISTS; that's how mIRC prescribes that it must be done,
  // and if all DLL coders did this, we would not need a mutex at all. NONE
  // of them do, so I am forced to follow suit to retain compatibility. bah!

  // map the shared memory onto our address space
  if ((ptr = (char *)MapViewOfFile(hmap, FILE_MAP_ALL_ACCESS, 0, 0, 0)) ==
   NULL) {
    CloseHandle(hmap);

    return;
  }

  // the creation and mapping of the virtual file done so far, can not lead to
  // synchronization problems on itself; we only need to protect actual access

  // so, create or open a mutex that will protect the shared memory access
  if ((hmutex = CreateMutex(NULL, FALSE, "mIRC_SendMessage")) != NULL) {

    // wait for this mutex to be released - if it is currently locked, wait!
    WaitForSingleObject(hmutex, INFINITE);

    // now that we hold the mutex, we can access the shared memory safely
    lstrcpyn(ptr, data, 1024);

    // and tell mIRC to execute our command
    SendMessage(hwnd, WM_MCOMMAND, 1, NULL);

    // release and destroy the mutex
    ReleaseMutex(hmutex);

    CloseHandle(hmutex);
  }

  // unmap and close the filemapping
  UnmapViewOfFile(ptr);

  CloseHandle(hmap);
}
