/*
 * ping.c - ICMP Ping DLL for mIRC
 * Copyright (c) 2005, D.C. van Moolenbroek, XISE <dc@xise.nl>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
  /<callback> <ip> [<status> <time> <ttl>]

  $dll(ping.dll,ping,ip [timeout [ttl]] [callback])
*/

#define WIN32_LEAN_AND_MEAN
#include <winsock2.h>
#include <windows.h>
#include <ipexport.h>

#include "thread.h"
#include "icmp.h"
#include "mirc.h"

#define DEF_TIMEOUT 1000
#define DEF_TTL 255

typedef struct {
  DWORD mVersion;
  HWND mHwnd;
  BOOL mKeep;
} LOADINFO;

typedef struct {
  HWND hwnd;
  DWORD ip;
  DWORD timeout;
  DWORD ttl;
  char callback[1];
} PingRequest;

static BOOL loaded = FALSE;

static char *InetNtoa(DWORD ip) {
  struct in_addr in;

  in.s_addr = ip;

  return inet_ntoa(in);
}

static DWORD WINAPI PingThread(LPVOID param) {
  PingRequest *req = (PingRequest *)param;
  IcmpResult res;
  char buf[1024];

  if (IcmpPing(req->ip, req->timeout, req->ttl, &res)) {
    wsprintf(buf, "/%s %s %u %u %u", req->callback, InetNtoa(res.ip),
     res.status, res.time, res.ttl);
  }
  else wsprintf(buf, "/%s %s", req->callback, InetNtoa(req->ip));

  SendMirc(req->hwnd, buf);

  LocalFree(req);

  return 0;
}

void __stdcall LoadDll(LOADINFO *loadinfo) {
  if (!InitThreads()) {
    loadinfo->mKeep = FALSE;

    return;
  }

  loaded = TRUE;
}

int __stdcall UnloadDll(int mTimeout) {
  if (mTimeout == 1 && GetNumThreads()) return 0;

  CleanupThreads();

  return 1;
}

static BOOL ScanNumber(char **buf, DWORD *result) {
  char *ptr = *buf;
  DWORD value;

  if (*ptr < '0' || *ptr > '9') return FALSE;

  for (value = 0; *ptr >= '0' && *ptr <= '9'; ptr++)
   value = value * 10 + *ptr - '0';

  if (*ptr && *ptr != ' ') return FALSE;

  if (*ptr) *ptr++ = 0;

  *buf = ptr;
  *result = value;
  return TRUE;
}

int __stdcall ping(HWND mWnd, HWND aWnd, char *data, char *parms, BOOL show,
 BOOL nopause) {
  PingRequest *req;
  HANDLE thread;
  DWORD id, ip, timeout, ttl, len;
  char *ptr;

  if (!loaded) return 1;

  CheckThreads();

  for (ptr = data; *ptr && *ptr != ' '; ptr++);

  if (*ptr == ' ') *ptr++ = 0;

  if ((ip = inet_addr(data)) == INADDR_NONE) return 1;

  if (!ScanNumber(&ptr, &timeout) || !timeout) timeout = DEF_TIMEOUT;

  if (!ScanNumber(&ptr, &ttl)) ttl = 0;

  if (!*ptr) ptr = ".signal PING";

  len = lstrlen(ptr);
  if (len > 850) return 1;

  req = (PingRequest *)LocalAlloc(LPTR, sizeof(PingRequest) + len);
  if (req == NULL) return 1;

  req->hwnd = mWnd;
  req->ip = ip;
  req->timeout = timeout;
  req->ttl = ttl;
  lstrcpy(req->callback, ptr);

  thread = CreateThread(NULL, 0, PingThread, req, 0, &id);

  if (thread == NULL) {
    LocalFree(req);
    return 1;
  }

  AddThread(thread);

  lstrcpy(data, "1");
  return 3;
}
