/*
 * thread.c - lazy thread management
 * Copyright (c) 2005, D.C. van Moolenbroek, XISE <dc@xise.nl>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include "thread.h"

#define INIT_SIZE 64

static HANDLE *thread;
static DWORD tnum, tsize;

BOOL InitThreads(void) {
  tnum = 0;
  tsize = INIT_SIZE;
  thread = LocalAlloc(LMEM_FIXED, tsize * sizeof(HANDLE));

  return thread != NULL;
}

void CleanupThreads(void) {
  DWORD i;

  CheckThreads();

  for (i = 0; i < tnum; i++)
   TerminateThread(thread[i], 1);

  LocalFree(thread);
}

void AddThread(HANDLE hThread) {
  if (tnum == tsize) {
    HANDLE *nthread =
     LocalReAlloc(thread, tsize * 2 * sizeof(HANDLE), LMEM_MOVEABLE);

    if (nthread == NULL) return;

    thread = nthread;
    tsize *= 2;
  }

  thread[tnum++] = hThread;
}

void CheckThreads(void) {
  DWORD status, i;

  for (i = 0; i < tnum; i++) {
    if (GetExitCodeThread(thread[i], &status) && status != STILL_ACTIVE) {
      CloseHandle(thread[i]);

      thread[i--] = thread[--tnum];
    }
  }

  if (tsize > INIT_SIZE && tnum * 2 < tsize) {
    HANDLE *nthread =
     LocalReAlloc(thread, tsize / 2 * sizeof(HANDLE), LMEM_MOVEABLE);

    if (nthread == NULL) return;

    thread = nthread;
    tsize /= 2;
  }
}

DWORD GetNumThreads(void) {
  CheckThreads();

  return tnum;
}
